/*
 * Copyright (C) 2004 BLADOX, s.r.o.  All rights reserved.
 * 
 * This file is part of an example program for Turbo. This example
 * program may be used, distributed and modified without limitation.
 *
 */

#include <config.h>
#include <tprog/tprog.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include "brum.h"
#include "i2cma.h"
#include "brum2i2c.h"
#include "brumuart.h"

#define MY_SLAVE_ADDRESS 0x77

void init_brum ()
{
  led_on (LED_ERR);
  hwd_set_default ();

  init_brumUart ();

  //PROG_DEBUG_VCC_ENABLE pin set it to log.0
  //(in EXT_SRAM_PROG_MODE we must apply power after RESETn=0)
  ext_cbi (EXT_PROG_DEBUG_VCC_ENABLE_PORT,
	   EXT_PROG_DEBUG_VCC_ENABLE_PORT_PIN);

  // wait for a moment
  delayMs (10);

  //I2C_ENABLE
  //set pin to log.1
  ext_sbi (EXT_I2C_ENABLE_PORT, EXT_I2C_ENABLE_PORT_PIN);

  delayMs (10);

  //BRUM_UART RX
  //let it set to input
  //set pin to log.1
  sbi (BRUM_RXD_PORT, BRUM_RXD_PORT_PIN);
  cbi (BRUM_RXD_DDR, BRUM_RXD_DDR_DD);

  //BRUM_UART TX
  //let it set to input
  //set pin to log.1
  sbi (BRUM_TXD_PORT, BRUM_TXD_PORT_PIN);
  cbi (BRUM_TXD_DDR, BRUM_TXD_DDR_DD);

  //BRUM_UART_ENABLE
  //set pin to log.1
  ext_sbi (EXT_BRUM_UART_ENABLE_PORT, EXT_BRUM_UART_ENABLE_PORT_PIN);

}

void turbo_handler (u8 action, USB_Data * data)
{
  u8 *buf = buf_A ();
  u16 i;
  u8 tmpC;
  u16 tmpI;

  switch (action)
  {
    case ACTION_APP_VERSION:
      data->buf[0] = VERSION_A;
      data->buf[1] = VERSION_B;
      data->buf[2] = VERSION_C;
      break;

    case ACTION_BRUM_INIT:
      reg_action (ACTION_SIG_UART1_RECV);
//      reg_action (ACTION_SIG_2WIRE_SERIAL);
      init_brum ();
      init_brum2i2c (MY_SLAVE_ADDRESS);

/*
      buf[0] = 'c';
      buf[1] = 'a';
      buf[2] = 'u';
      buf[3] = '\r';
      buf[4] = '\n';
      brumUartTxSendData (buf, 5);
*/

      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_SIG_UART1_RECV:
      brum_uart_irq ();
      break;

    case ACTION_BRUM_SELECT_SLAVE:
      brum2i2c_selectSlave (data->buf[0]);
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_GET_SLAVE:
      brum2i2c_getSlave (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_TX_DATA:
    case ACTION_BRUM_RX_DATA:
    case ACTION_BRUM_SELECT_REG:
    case ACTION_BRUM_READ_RX_FIFO_DATA:
    case ACTION_BRUM_WRITE_TX_FIFO_DATA:
      break;

    case ACTION_BRUM_READ_RX_FIFO_STATUS:
      brum2i2c_read_rxfifo_status (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_READ_TX_FIFO_STATUS:
      brum2i2c_read_txfifo_status (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_READ_RX_FIFO_CMD:
      brum2i2c_read_rxfifo_cmd (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_WRITE_RX_FIFO_CMD:
      brum2i2c_write_rxfifo_cmd (data->buf[0]);
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_READ_TX_FIFO_CMD:
      brum2i2c_read_txfifo_cmd (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_WRITE_TX_FIFO_CMD:
      brum2i2c_write_txfifo_cmd (data->buf[0]);
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_READ_CONTROL:
      brum2i2c_read_control (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_WRITE_CONTROL:
      brum2i2c_write_control (data->buf[0]);
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_READ_RXCHARS:
      brum2i2c_read_rxchars (&tmpI);
      usb_send (NO_ERROR, 2, (u8 *) (&tmpI));
      break;

    case ACTION_BRUM_READ_TXCHARS:
      brum2i2c_read_txchars (&tmpI);
      usb_send (NO_ERROR, 2, (u8 *) (&tmpI));
      break;

    case ACTION_BRUM_SET_BRUM_I2C_INT_STATUSn:
      if (data->buf[0] != 0)
      {				//BRUM_I2C_INT_STATUSn   //set pin to log.1
	sbi (BRUM_I2C_INT_STATUSn_ENABLE_PORT,
	     BRUM_I2C_INT_STATUSn_ENABLE_PORT_PIN);
      }
      else
      {				//BRUM_I2C_INT_STATUSn   //set pin to log.0
	cbi (BRUM_I2C_INT_STATUSn_ENABLE_PORT,
	     BRUM_I2C_INT_STATUSn_ENABLE_PORT_PIN);
      }
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_SET_BRUM1_SEL:
      if (data->buf[0] != 0)
      {				//BRUM1_SEL   //set pin to log.1
	sbi (BRUM1_SEL_PORT, BRUM1_SEL_PORT_PIN);
      }
      else
      {				//BRUM1_SEL   //set pin to log.0
	cbi (BRUM1_SEL_PORT, BRUM1_SEL_PORT_PIN);
      }
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_SET_BRUM2_SEL:
      if (data->buf[0] != 0)
      {				//BRUM2_SEL   //set pin to log.1
	sbi (BRUM2_SEL_PORT, BRUM2_SEL_PORT_PIN);
      }
      else
      {				//BRUM2_SEL   //set pin to log.0
	cbi (BRUM2_SEL_PORT, BRUM2_SEL_PORT_PIN);
      }
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_READ_DEBUG_CHARS:
      brum2i2c_read_debug_chars (&tmpI);
      usb_send (NO_ERROR, 2, (u8 *) (&tmpI));
      break;

    case ACTION_BRUM_READ_DEBUGFIFO_DATA:
      tmpI = data->buf[0] | (data->buf[1] << 8);
      for (i = 0; i < tmpI; i++)
      {
	brum2i2c_read_debug_data (&(buf[i]), 1);
      }
      usb_send (NO_ERROR, tmpI, buf);
      break;

    case ACTION_BRUM_RXFIFO_UPLOAD_DATA:
      tmpI = data->buf[0] | (data->buf[1] << 8);
      brumUartTxSendData (&(data->buf[2]), tmpI);
      usb_send (NO_ERROR, 0, NULL);
      break;

    case ACTION_BRUM_TXFIFO_DOWNLOAD_DATA:
      tmpI = data->buf[0] | (data->buf[1] << 8);
      for (i = 0; i < tmpI; i++)
	buf[i] = brumUartRxGetChar ();
      usb_send (NO_ERROR, tmpI, buf);
      break;

    case ACTION_BRUM_READ_RSSI:
      brum2i2c_read_rssi (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_READ_ID:
      brum2i2c_read_id (&tmpC);
      usb_send (NO_ERROR, 1, &tmpC);
      break;

    case ACTION_BRUM_APPS_INIT:
/*
-> ACTION_BRUM_APPS_INIT
<- ret
*/
      buf[0] = action;
      brumUartTxSendData (buf, 1);
      for (i = 0; i < 1; i++)
      {
	while (brumUartRxChars () == 0);
	buf[i] = brumUartRxGetChar ();
      }

      usb_send (NO_ERROR, 1, buf);

      break;
    case ACTION_BRUM_APPS_VERSION:
/*
-> ACTION_BRUM_APPS_VERSION
<- 3x version
<- 8x mac
<- 14 sn
*/

      buf[0] = action;
      brumUartTxSendData (buf, 1);
      for (i = 0; i < 3 + 8 + 14; i++)
      {
	while (brumUartRxChars () == 0);
	buf[i] = brumUartRxGetChar ();
      }
      usb_send (NO_ERROR, 3 + 8 + 14, buf);
      break;

    case ACTION_BRUM_APPS_LIST:
/*
->ACTION_BRUM_APPS_LIST
<- len_h, len_l
<- data
*/

      buf[0] = action;
      brumUartTxSendData (buf, 1);

      for (i = 0; i < 3; i++)
      {
	while (brumUartRxChars () == 0);
	buf[i] = brumUartRxGetChar ();
      }

      tmpI = buf[2] | (((u16) buf[1]) << 8);

      for (i = 0; i < tmpI; i++)
      {
	while (brumUartRxChars () == 0);
	buf[3 + i] = brumUartRxGetChar ();
      }

      usb_send (NO_ERROR, 3 + tmpI, buf);

      break;

    case ACTION_BRUM_APPS_UPLOAD:
/*
-> ACTION_BRUM_APPS_UPLOAD:
-> len_h, len_l
-> data
<- ret
*/

      i = data->buf[0];
      data->buf[0] = action;
      brumUartTxSendData (data->buf, 1);

      while (brumUartRxChars () == 0);
      brumUartRxGetChar ();	// wait for ready

      data->buf[0] = i;
      brumUartTxSendData (data->buf, data->len);

#if 1
/*
	buf[0]=0x00;
	buf[1]=0x01;
	buf[2]=0x00;
      brumUartTxSendData (buf, 3);
*/

/*
      tmpI = (u16)data->buf[1] | (((u16)data->buf[0]) << 8);
      brumUartTxSendData (data->buf, tmpI+2);

led_on(tmpI+2);
*/

      for (i = 0; i < 1; i++)
      {
	while (brumUartRxChars () == 0);
	buf[i] = brumUartRxGetChar ();
      }

      usb_send (NO_ERROR, 1, buf);
#endif
//      usb_send (NO_ERROR, data->len, data->buf);

      break;
    case ACTION_BRUM_APPS_REMOVE:
/*
-> ACTION_BRUM_APPS_REMOVE:
-> app nr
<- ret
*/

      i = buf[0];
      buf[0] = action;
      brumUartTxSendData (buf, 1);
      buf[0] = i;

      brumUartTxSendData (data->buf, data->len);

      for (i = 0; i < 1; i++)
      {
	while (brumUartRxChars () == 0);
	buf[i] = brumUartRxGetChar ();
      }

      usb_send (NO_ERROR, 1, buf);

      break;

    default:
      break;
  }
}
