#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
//#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <tproglib.h>
#include "../module-src/brum.h"

#ifdef WIN32_MINGW
#include <windows.h>
#endif

#define CABLE_UPLOAD_APP_START  0x00
#define CABLE_UPLOAD_APP_CONT   0x01
#define CABLE_UPLOAD_APP_END    0x02

#define NO_ERROR                0x00
#define ERR_NO_PROGMEM          0x01
#define ERR_NO_RAM              0x02
#define ERR_NO_APP_SLOT         0x04
#define ERR_NO_EEPROM           0x07
#define ERR_BAD_FILE            0x0c

#define ERR_APP_NOT_FOUND	0x81

/*
lock?
module 1/2
*/

void help ()
{
  printf ("Usage: brum_apps [args]\n");
  printf ("-a application\tuse application, default is %s\n", APP_BRUM);
  printf ("-d device\tuse USB device\n");
  printf ("-i\t\tinit eeprom\n");
  printf ("-u file\t\tupload file\n");
  printf ("-m brum\t\tbrum 1/2, default 1\n");
  printf ("-r name\t\tremove application\n");
//  printf ("-a app\t\tsend action to app\n");
//  printf ("-x action\taction to send\n");
//  printf ("-s data\t\tdata to send\n");
  printf ("-h\t\thelp\n");
  printf ("-v\t\tverbose\n");
  exit (-1);
}

u8 app_open ()
{
  u16 i, j;
  u8 *t;
  u8 res[1000];
  b32 f;
  u16 len;

  u8 answer[2048];
  u16 answerLen;

  printf (">app_open\n");

  t = res;

  len = 1;
  *t = len >> 8;
  t++;
  *t = len;
  t++;

  *t = CABLE_UPLOAD_APP_START;
  t++;

  len = t - res;
  usb_send (ACTION_BRUM_APPS_UPLOAD, len, res, &answerLen, answer);

  i = answer[0];

  printf ("<app_open %d\n", i);

  return i;
}

u8 app_append (u8 * data, u16 pos, u8 l)
{
  u16 i = NO_ERROR;
  u8 res[1000];
  u8 *t = res;
  u16 len;
  u8 *x;
  u8 answer[2048];
  u16 answerLen;

  printf (">app_append %d %d\n", pos, l);
  len = l + 2;
  *t = len >> 8;
  t++;
  *t = len;
  t++;

  *t = CABLE_UPLOAD_APP_CONT;
  t++;
  *t = l;
  t++;
  for (i = 0; i < l; i++)
  {
    *t = data[pos + i];
    t++;
  }

/*
for(x=res;x!=t;x++)printf("%02X ", *x);
printf("\n");fflush(stdout);
*/

  len = t - res;

  usb_send (ACTION_BRUM_APPS_UPLOAD, len, res, &answerLen, answer);

  i = answer[0];

  printf ("<app_append %d\n", i);
  return i;
}

u8 app_close (u8 * name)
{
  u16 i = NO_ERROR;
  u8 res[1000];
  u8 *t = res;
  u8 l = strlen (name);
  u16 len;
  u8 answer[2048];
  u16 answerLen;

  printf (">app_close\n");
  len = l + 2;
  *t = len >> 8;
  t++;
  *t = len;
  t++;

  *t = CABLE_UPLOAD_APP_END;
  t++;

  for (i = 0; i < l; i++)
  {
    *t = name[i];
    t++;
  }
  *t = '\0';
  t++;

  len = t - res;
  usb_send (ACTION_BRUM_APPS_UPLOAD, len, res, &answerLen, answer);

  i = answer[0];
  printf ("<app_close %d\n", i);

  return i;
}

int main (int argc, char **argv)
{
  u16 i;
  u16 ii;
  u16 j;

  u16 answerSizeTotal;
  u16 flashPageNumber;

  u8 *app_name = APP_BRUM;
  u8 answer[2048];
  u8 param[2048];
  u16 answerLen;
  u16 paramLen;

  int fdstdin;
  int flags;
  int bytes_read;
  u16 len;

  int result;
  u8 *dev = USBDEVICE;
  u8 ef[] = { 0x7f, 0x20 };
  u16 sw1sw2;
  u8 buf[1000];
  u8 init_eeprom = 0;
  u8 *upload = NULL;
  u8 action = 0x00;
  u8 app = 0xff;
  u8 *send_data = NULL;
  USB_Data usb_data;
  u8 *remove = NULL;
  u8 sizeof_App_vector = 15;
  u8 ver_prog_A;
  u8 ver_A, ver_B, ver_C;
  u8 brum = 1;

  do
  {
    result = getopt (argc, argv, "hd:iu:a:x:s:vr:m:");
    switch (result)
    {
      case 's':
	send_data = optarg;
	break;
      case 'u':
	upload = optarg;
	break;
      case 'r':
	remove = optarg;
	break;
      case 'a':
	app_name = optarg;
	break;
      case 'x':
	action = atoi (optarg);
	break;
      case 'm':
	brum = atoi (optarg);
	break;
      case 'i':
	init_eeprom = 1;
	break;
      case 'd':
	dev = optarg;
	break;
      case 'h':
	help ();
	break;
      case 'v':
	verbose_inc ();
	break;
    }
  }
  while (result > 0);

  if (brum != 1 && brum != 2)
    help ();

/*
  usb_data.buf = send_data;
  if (send_data)
    usb_data.len = strlen (send_data);
  else
    usb_data.len = 0;
*/

  open_device (dev);

  if (select_app (app_name) != NO_ERROR)
  {
    close_device ();
    fprintf (stderr, "Cannot open application %s\n", app_name);
    exit (-1);
  }

  usb_send (ACTION_BRUM_INIT, 0, NULL, &answerLen, answer);
  printf ("BRUM VCC enabled\n");

  param[0] = 0;
  usb_send (ACTION_BRUM_SET_BRUM1_SEL, 1, param, &answerLen, answer);
  printf ("BRUM1_SEL set to 0\n");

  param[0] = 0;
  usb_send (ACTION_BRUM_SET_BRUM2_SEL, 1, param, &answerLen, answer);
  printf ("BRUM2_SEL set to 0\n");

  if (brum == 1)
  {
    param[0] = 1;
    usb_send (ACTION_BRUM_SET_BRUM1_SEL, 1, param, &answerLen, answer);
    printf ("BRUM1_SEL set to 1\n");
  }
  else
  {
    param[0] = 1;
    usb_send (ACTION_BRUM_SET_BRUM2_SEL, 1, param, &answerLen, answer);
    printf ("BRUM2_SEL set to 1\n");
  }

#if 0
  if (app != 0xff)
  {
    printf ("Send action %d to app %d\n", action, app);
    select_app_nr (app);
    i = usb_send (action, usb_data.len, usb_data.buf, &len, buf);
    printf ("Status: %02X\n", i);
    printf ("Len: %d\n", len);
    for (i = 0; i < len; i++)
    {
      if (i != 0)
	printf (".");
      printf ("%02X", buf[i]);
    }
    printf ("\n");
  }
#endif

  if (remove)
  {
    u8 nr_apps;
    u16 av_handler;
    u16 av_data;
    u16 av_text_len;
    u16 av_data_len;
    u16 av_bss_len;
    u8 av_text_start;
    u16 av_ram_start;
    u16 av_name;
    u8 *av;
    u8 *p;
    u8 app_nr = 0xff;

    usb_send (ACTION_BRUM_APPS_VERSION, 0, NULL, &answerLen, answer);
    ver_prog_A = ver_A = answer[0];
    ver_B = answer[1];
    ver_C = answer[2];

    ver_prog_A = 3;

    usb_send (ACTION_BRUM_APPS_LIST, 0, NULL, &answerLen, answer);

    len = answer[1];
    len <<= 8;
    len += answer[2];
    for (i = 0; i < len; i++)
      buf[i] = answer[3 + i];

    nr_apps = buf[6];
    p = &buf[7];

    if (ver_prog_A >= 3)
      sizeof_App_vector = 18;

    for (i = 0; i < nr_apps; i++)
    {
      av = p;
      av_handler = (((u16) av[0]) << 8) | av[1];
      av_data = (((u16) av[2]) << 8) | av[3];
      av_text_len = (((u16) av[4]) << 8) | av[5];
      av_data_len = (((u16) av[6]) << 8) | av[7];
      av_bss_len = (((u16) av[8]) << 8) | av[9];
      av_text_start = av[10];
      av_ram_start = (((u16) av[11]) << 8) | av[12];
      if (ver_prog_A >= 3)
      {
	ver_A = av[13];
	ver_B = av[14];
	ver_C = av[15];
	av_name = (((u16) av[16]) << 8) | av[17];
      }
      else
	av_name = (((u16) av[13]) << 8) | av[14];

      p += sizeof_App_vector;

      if (av_name)
      {
	if (strcmp (p, remove) == 0)
	{
	  app_nr = i;
	  break;
	}
	else
	{
	  while (*p)
	  {
	    p++;
	  }
	  p++;
	}
      }
    }

    if (app_nr == 0xff)
    {
      close_device ();
      fprintf (stderr, "Cannot remove app %s, not installed\n", remove);
      exit (-1);
    }

    param[0] = app_nr;

    usb_send (ACTION_BRUM_APPS_REMOVE, 1, param, &answerLen, answer);
  }
  else if (upload)
  {
    u8 *s_name;
    u8 *d_name;
    int s_size;
    u8 *s_buf;
    struct stat _st;
    FILE *fs;
    int ret;
    u8 f;
    int pos;

    s_name = upload;
//    printf ("SRC %s\n", s_name);

#if 0
    if (select_app (s_name) == NO_ERROR)
    {
      close_device ();
      fprintf (stderr, "Cannot upload app %s, already installed\n", s_name);
      exit (-1);
    }
#endif

    ret = stat (s_name, &_st);
    if (ret < 0)
    {
      printf ("Cannot stat %s\n", s_name);
      exit (-1);
    }
//    printf ("SIZE %d\n", _st.st_size);
    s_size = _st.st_size;
    s_buf = malloc (s_size);
#ifndef WIN32_MINGW
// Why the !@#$%^& it doesn't work on MINGW?
    fs = fopen (s_name, "r");
    for (i = 0; i < s_size; i++)
      s_buf[i] = fgetc (fs);
    fclose (fs);
#else
    {
      HANDLE *f1;
      int n_in;

      f1 = CreateFile (s_name, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);

      if (f1 == INVALID_HANDLE_VALUE)
      {
	printf ("Cannot open file %s (error %d)\n", s_name, GetLastError ());
	exit (-1);
      }

      ReadFile (f1, s_buf, s_size, (DWORD *) (&n_in), NULL);

      CloseHandle (f1);
    }
#endif

/*
printf("BUF\n");
    for (i = 0; i < s_size; i++)
{
printf("%02X ", s_buf[i]);
if(i%20==0)printf("\n");
}
printf("\n");fflush(stdout);
*/
    f = app_open ();
    if (f)
    {
      printf ("ERROR %d\n", f);
      goto E_RR;
    }

    pos = 0;

    while (pos < s_size)
    {
      len = s_size - pos;
      if (len > 128)
	len = 128;
      f = app_append (s_buf, pos, len);
      if (f)
      {
	printf ("ERROR %d\n", f);
	goto E_RR;
      }
      pos += 128;
    }

    f = app_close (s_name);
    if (f)
      printf ("ERROR %d\n", f);

  E_RR:
  }
  else if (init_eeprom)
  {
    printf ("Clearing BRUM EEPROM, uninstalling apps\n");
    usb_send (ACTION_BRUM_APPS_INIT, 0, NULL, &answerLen, answer);
  }
  else
  {
    u8 nr_apps;
    u16 av_handler;
    u16 av_data;
    u16 av_text_len;
    u16 av_data_len;
    u16 av_bss_len;
    u8 av_text_start;
    u16 av_ram_start;
    u16 av_name;
    u8 *av;
    u8 *p;

#if 1
    usb_send (ACTION_BRUM_APPS_VERSION, 0, NULL, &answerLen, answer);
    ver_prog_A = ver_A = answer[0];
    ver_B = answer[1];
    ver_C = answer[2];

    ver_prog_A = 3;
    if (ver_prog_A >= 3)
      sizeof_App_vector = 18;

    printf ("BRUM kernel version: %d.%d.%d\n", ver_A, ver_B, ver_C);
    printf ("SN:\t");
    for (i = 0; i < 14; i++)
      printf ("%c", answer[3 + 8 + i]);
    printf ("\n");
    printf ("MAC:\t");
    for (i = 0; i < 8; i++)
    {
      printf ("%d", answer[3 + i]);
      if (i != 7)
	printf (".");
    }
    printf ("\n");

#endif

    printf ("BRUM application list:\n");

    usb_send (ACTION_BRUM_APPS_LIST, 0, NULL, &answerLen, answer);

    len = answer[1];
    len <<= 8;
    len += answer[2];
    for (i = 0; i < len; i++)
      buf[i] = answer[3 + i];

/*
    printf ("RAM END %02X%02X\n", buf[0], buf[1]);
    printf ("PROG END %02X%02X%02X%02X\n", buf[2], buf[3],
	    buf[4], buf[5]);
*/

    nr_apps = buf[6];
    p = &buf[7];

    for (i = 0; i < nr_apps; i++)
    {
      u8 ver_A, ver_B, ver_C;

//      printf ("%2d: ", i);
      av = p;
      av_handler = (((u16) av[0]) << 8) | av[1];
      av_data = (((u16) av[2]) << 8) | av[3];
      av_text_len = (((u16) av[4]) << 8) | av[5];
      av_data_len = (((u16) av[6]) << 8) | av[7];
      av_bss_len = (((u16) av[8]) << 8) | av[9];
      av_text_start = av[10];
      av_ram_start = (((u16) av[11]) << 8) | av[12];

      if (ver_prog_A >= 3)
      {
	ver_A = av[13];
	ver_B = av[14];
	ver_C = av[15];
	av_name = (((u16) av[16]) << 8) | av[17];
      }
      else
	av_name = (((u16) av[13]) << 8) | av[14];

      p += sizeof_App_vector;

/*
      printf ("Handler:\t %05X\n", av_handler * 2);
      printf ("DATA:\t\t %04X\n", av_data);
      printf ("TEXT len:\t %04X\n", av_text_len);
      printf ("DATA len:\t %04X\n", av_data_len);
      printf ("BSS len:\t %04X\n", av_bss_len);
      printf ("TEXT start:\t %02X\n", av_text_start);
      printf ("RAM start:\t %04X\n", av_ram_start);
      printf ("Name:\t\t %04X ", av_name);
*/

      if (av_name)
      {
	printf ("%2d: ", i);
	while (*p)
	{
	  printf ("%c", *p);
	  p++;
	}
	p++;
	printf ("\t\t\t%d.%d.%d", ver_A, ver_B, ver_C);
	printf ("\n");
      }

/*
      for (j = 0; j < sizeof_App_vector; j++)
      {
	printf ("%02X ", av[j]);
      }

      printf ("\n");
      printf ("\n");
*/
    }
/*
    printf ("\n");

    for (i = 0; i < len; i++)
    {
      if (i != 0)
	printf (".");
      printf ("%02X", buf[i]);
    }
    printf ("\n");
*/
//  usb_send (ACTION_BRUM_APPS_VERSION, 0, NULL, &answerLen, answer);
  }

  close_device ();
}
